//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.jameica.depot.finanzen;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;

import de.frankmuenster.jameica.depot.finanzen.boerseffm.FfmWertpapierHistory;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooWertpapierHistory;
import de.willuhn.logging.Logger;

/**
 * 
 * @author Frank Mnster
 * 
 */
public class WertpapierHistory {

	/**
	 * DatenAbfrage erfolgreich
	 */
	protected transient boolean dataRequestOK;
	
	/**
	 * Anfangsdatum der Werte
	 */
	protected final Date fromDate;
	
	/**
	 * Das Yahoo Symbol
	 */
	protected final transient String symbol;
	
	/**
	 * Der Name fr die Brese Frankfurt
	 */
	protected final transient String name;

	/**
	 * Enddatum der Werte
	 */
	protected final Date toDate;
	
	/**
	 * Datenquelle
	 */
	private transient String dataSource;
	
	/**
	 * Liefert eine Liste mit den historischen Werten des Wertpapiers
	 */
	protected transient List<HistoryValueBean> wphv = new ArrayList<HistoryValueBean>();

	/**
	 * Konstruktor mit Angabe des Yahoo-Symbols und/oder das FFMB Names
	 * 
	 * @param symbol 
	 * @param name 
	 * 
	 * @throws FinanceException 
	 */
	public WertpapierHistory(final String symbol, final String name) throws FinanceException {

		this.symbol = symbol;
		this.name = name;
		this.dataRequestOK = false;
		this.dataSource = StringUtils.EMPTY;

		// Erst die Daten laden
		try {
			final YahooWertpapierHistory ywh = new YahooWertpapierHistory(symbol);
			wphv = ywh.loadData();
			this.dataSource = "Yahoo"; //$NON-NLS-1$
		} catch (FinanceException ex) {
			// Hier nur loggen, wir haben ja noch die Brse FFM
			Logger.error(ex.getLocalizedMessage());
		}
		
		if (wphv.isEmpty() && name != null) {
			try {
			// probieren wir es mal mit der Brse FFM
			final FfmWertpapierHistory fwh = new FfmWertpapierHistory(name);
			wphv = fwh.loadData();
			this.dataSource = "Brse Frankfurt"; //$NON-NLS-1$
			} catch (FinanceException ex) {
				// Nun werfen wir die Exception doch
				Logger.error(ex.getLocalizedMessage());
				this.dataSource = StringUtils.EMPTY;
				throw ex;
			}
		}

		if (wphv.isEmpty()) {
			this.dataSource = StringUtils.EMPTY;
			throw new FinanceException("Keine historischen Daten gefunden", FinanceExceptionType.DATA_NOT_FOUND); //$NON-NLS-1$
		}
		this.dataRequestOK = true;

		// Dann sortieren.
		Collections.sort(this.wphv);

		// und das Anfangs- und Enddatum setzten
		this.fromDate = this.wphv.get(0).getDate();
		this.toDate = this.wphv.get(this.wphv.size() - 1).getDate();
	}

	/**
	 * Liefert das Anfangsdatum
	 * 
	 * @return fromDate
	 */
	public Date getFromDate() {
		return fromDate;
	}

	/**
	 * Liefert eine Liste mit den historischen Werten des Wertpapiers.
	 * 
	 * @return wphv Liste mit den HistoryValueBean
	 */
	public List<HistoryValueBean> getHistoryValues() {
		return wphv;
	}

	/**
	 * Liefert das Yahoo-Symbol
	 * 
	 * @return symbol
	 */
	public String getSymbol() {
		return symbol;
	}

	/**
	 * Liefert die historischen Werte f&uuml;r eine angegeben Zeitraum.
	 * 
	 * @param fromDate
	 *            das Anfangsdatum
	 * @param toDate
	 *            das Enddatum
	 * @return Die Liste mit HistoryValueBean
	 */
	public List<HistoryValueBean> getTimeRangeValues(final Date fromDate, final Date toDate) {

		final List<HistoryValueBean> rangeValues = new ArrayList<HistoryValueBean>();

		for (final HistoryValueBean value : wphv) {
			final long dateInt = value.getDate().getTime();
			if (dateInt > fromDate.getTime() && dateInt < toDate.getTime()) {
				rangeValues.add(value);
			}
		}

		return rangeValues;
	}

	/**
	 * Liefert die historischen Werte f&uuml;r ein angegebenes Datum. Falls kein
	 * Daten f&uuml; das angegeben Datum zur Verf&uuml;gung stehen, wird der
	 * n&auml;chste davorliegende Handelstag genommen. Liegt kein Tage vor
	 * DATUM, werden die Daten des ersten zur Verf&uuml;gung stehenden Tages
	 * verwendet. Liegen gar keine Daten vor wir NULL zur&uuml;ckgegeben.
	 * 
	 * @param date
	 *            das Datum
	 * @return Der HistoryValueBean
	 */
	public HistoryValueBean getValuesAtDate(final Date date) {

		HistoryValueBean returnValue = null;

		if (!wphv.isEmpty()) {
			returnValue = wphv.get(0);

			for (final HistoryValueBean value : wphv) {

				final Date dateValue = value.getDate();
				if (DateUtils.isSameDay(date, dateValue)) {
					// Gesuchtes Datum gefunden
					returnValue = value;
					break;
				} else if (date.before(dateValue)) {
					// Wir sind am gesuchten Datum vorbei
					break;
				}
				returnValue = value;
			}
		}
		return returnValue;
	}

	/**
	 * Liefert das Enddatum
	 * 
	 * @return toDate
	 */
	public Date getToDate() {
		return toDate;
	}

	/**
	 * War der Datenabruf OK.
	 * 
	 * @return the dataRequestOK
	 */
	public Boolean isDataRequestOK() {
		return dataRequestOK;
	}
	
	/**
	 * Liefert die Datenquelle
	 * @return Datenquelle (Yahoo, Brese FFM)
	 */
	public String getDataSource() {
		return this.dataSource;
	}


}