//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.jameica.depot.finanzen.boerseffm;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.finanzen.FinanceException;
import de.frankmuenster.jameica.depot.finanzen.FinanceExceptionType;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;

/**
 * @author munstfr
 * 
 */
public final class FfmWebAPI {
	
	/**
	 * Standartantwort, falls nichts gefunden wurde.
	 */
	private final static String NOT_A_RESOURCE = "NaR"; //$NON-NLS-1$

	/**
	 * Liste der schon abgefragte URLs
	 */
	private final transient Map<URL, String> cacheUrlResponse = new ConcurrentHashMap<URL, String>();
	
	
	/**
	 * Unser Instanz
	 */
	private static FfmWebAPI instance;

	/**
	 * Privater Konstruktor
	 */
	private FfmWebAPI() {

	}

	/**
	 * Liefert die YahooFinanceAPI Instanz. Wird bei bedarf erstellt.
	 * 
	 * @return die YahooFinanceAPI-Instanz
	 */
	public static FfmWebAPI getInstance() {
		synchronized (FfmWebAPI.class) {
			if (FfmWebAPI.instance == null) {
				FfmWebAPI.instance = new FfmWebAPI();
			}
		}
		return FfmWebAPI.instance;
	}

	/**
	 * Liefert die Antwort eines Web-Services Aufrufs.
	 * 
	 * @param url
	 *            des Webservices.
	 * @return die Antwort des Web Service, oder <code>"NaR"</code> falls mit
	 *         dem Aufruf alles in Ordnung war, aber keine Antwort kam.
	 * @throws FinanceException
	 *             falls w&auml;hrend des Aufrufs etwas schief ging.
	 */
	public String getResonseGETRequest(final URL url) throws FinanceException {
		return getResonseGETRequest(url, false);
	}

	/**
	 * Liefert die Antwort eines Web-Services Aufrufs.
	 * 
	 * @param url
	 *            des Webservices.
	 * @param useCache
	 *            <code>true</code> falls der interne Cache verwendet werden
	 *            soll, sonst <code>false</code>.
	 * @return die Antwort des Web Service, oder <code>"NaR"</code> falls mit
	 *         dem Aufruf alles in Ordnung war, aber keine Antwort kam.
	 * @throws FinanceException
	 *             falls w&auml;hrend des Aufrufs etwas schief ging.
	 */
	public String getResonseGETRequest(final URL url, final boolean useCache) throws FinanceException {

		String response;

		if (useCache && cacheUrlResponse.containsKey(url)) {
			Logger.debug(Settings.getI18n().tr("YahooFinanceAPI.loadfromCacheInfo")); //$NON-NLS-1$
			response = cacheUrlResponse.get(url);
		} else {

			response = NOT_A_RESOURCE;

			try {
				
				if (Level.DEBUG.equals(Logger.getLevel())) {
					Logger.debug(Settings.getI18n().tr(
							"YahooFinanceAPI.url", url.toString())); //$NON-NLS-1$
				}
				final HttpURLConnection conn = (HttpURLConnection) url
						.openConnection();

				conn.setRequestMethod("GET"); //$NON-NLS-1$
				conn.setConnectTimeout(2000);
				//conn.setRequestProperty("Accept", "application/json"); //$NON-NLS-1$ //$NON-NLS-2$
				conn.setRequestProperty("Accept-Charset", "UTF-8"); //$NON-NLS-1$ //$NON-NLS-2$
				conn.setRequestProperty("User-Agent", "JaDepot"); //$NON-NLS-1$ //$NON-NLS-2$

				Logger.info(conn.getResponseMessage());
				
				if (conn.getResponseCode() != HttpURLConnection.HTTP_OK) {
					throw new FinanceException(Settings.getI18n().tr(
							"YahooFinanceAPI.httpError" //$NON-NLS-1$
							, Integer.toString(conn.getResponseCode())),
							FinanceExceptionType.NETWORK);
				}

				final BufferedReader bufferReader = new BufferedReader(
						new InputStreamReader(conn.getInputStream(), "UTF-8")); //$NON-NLS-1$

				// System.out.println("Output from Server .... \n");
				final StringBuffer sBuffer = new StringBuffer(""); //$NON-NLS-1$
				String output;
				while ((output = bufferReader.readLine()) != null) { // NOPMD
					// System.out.println(output);
					sBuffer.append(output).append(System.lineSeparator());
				}

				response = sBuffer.toString();
				
				conn.disconnect();

				if (useCache) {
					cacheUrlResponse.put(url, response);
				}
				

			} catch (MalformedURLException e) {
				throw (FinanceException) new FinanceException(Settings.getI18n().tr(
						"YahooFinanceAPI.malformedUrl", e.getMessage()), //$NON-NLS-1$
						FinanceExceptionType.NETWORK).initCause(e);

			} catch (IOException e) {
				throw (FinanceException) new FinanceException(Settings.getI18n().tr(
						"YahooFinanceAPI.ioException", e.getMessage()), //$NON-NLS-1$
						FinanceExceptionType.NETWORK).initCause(e);
			}
		}
		return response;

	}

}
