//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.jameica.depot.finanzen.boerseffm;

import java.io.IOException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

import com.fasterxml.jackson.databind.ObjectMapper;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.finanzen.FinanceException;
import de.frankmuenster.jameica.depot.finanzen.FinanceExceptionType;
import de.frankmuenster.jameica.depot.finanzen.HistoryValueBean;
import de.frankmuenster.jameica.depot.finanzen.IWertpapierHistory;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.json.ChartData;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.json.ChartDataResponse;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.json.Instrument;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.json.InstrumentType;
import de.frankmuenster.jameica.depot.finanzen.boerseffm.json.SearchResponse;
import de.willuhn.logging.Logger;

/**
 * L&auml;d historisch Werte von Frankfurter B&ouml;se.
 * 
 * @author Frank Mnster
 * 
 */
public class FfmWertpapierHistory implements IWertpapierHistory {

	/**
	 * Jackson Mapper
	 */
	private static final ObjectMapper MAPPER = new ObjectMapper();

	/**
	 * B&OUML;rese FFM URL
	 */
	private static final String MAIN_URL = "http://www.boerse-frankfurt.de/"; //$NON-NLS-1$

	/**
	 * B&OUML;rese FFM URL proxy
	 */
	private static final String PROXY_URL = "http://proxy.boerse-frankfurt.de/"; //$NON-NLS-1$

	/**
	 * Such URL
	 */
	public static final String SEARCH_URL = MAIN_URL + "Ajax/SearchController_JsonSuggest?Query="; //$NON-NLS-1$

	/**
	 * Such URL Parameter
	 */
	public static final String SEARCH_URL_PARAMS = "&SearchType=false"; //$NON-NLS-1$

	/**
	 * Daten URL
	 */
	private static final String DATA_URL = PROXY_URL + "cst/BoerseFrankfurt/Share/chart.json?instruments="; //$NON-NLS-1$

	/**
	 * Daten URL Parameter
	 */
	private static final String DATA_URL_PARAMS = "&period=Maximum"; //$NON-NLS-1$

	/**
	 * Zahl eins
	 */
	private static final int EXACT_ONE = 1;

	/**
	 * Der eindeutige Name
	 */
	private final transient String name;

	/**
	 * Konstruktor mit Angabe des Names
	 * 
	 * @param name
	 */
	public FfmWertpapierHistory(final String name) {
		this.name = name;
	}

	/**
	 * @see IWertpapierHistory#loadData()
	 */
	@Override
	public List<HistoryValueBean> loadData() throws FinanceException {

		List<HistoryValueBean> wphvList = new ArrayList<>();

		final URL url = getMainUrl();
		if (url != null) {
			final String responseString = FfmWebAPI.getInstance().getResonseGETRequest(url, true);

			final int pos = responseString.indexOf("preLoadedSeries[0] = {name:"); //$NON-NLS-1$

			if (pos > 0) {

				Logger.debug("Super, gefunden!!"); //$NON-NLS-1$
				final int start = responseString.indexOf("identifier:"); //$NON-NLS-1$
				final int end = responseString.indexOf("',language:", pos); //$NON-NLS-1$
				final String instrument = responseString.substring(start + 13, end);
				wphvList = readData(instrument);

			}
		}
		return wphvList;

	}

	/**
	 * Ermittelt die konkrete URL zu dem Wertpapier.
	 * 
	 * @return URL zu dem Instrument
	 * @throws FinanceException
	 */
	private URL getMainUrl() throws FinanceException {

		URL url = null;
		try {
			final String encodedName = URLEncoder.encode(name, java.nio.charset.StandardCharsets.UTF_8.name());
			StringBuilder urlSB = new StringBuilder(130);
			urlSB.append(SEARCH_URL).append(encodedName).append(SEARCH_URL_PARAMS);
			final URL searchUrl = new URL(urlSB.toString());

			final String responseString = FfmWebAPI.getInstance().getResonseGETRequest(searchUrl, true);

			final SearchResponse searchResponse = FfmWertpapierHistory.MAPPER.readValue(responseString,
					SearchResponse.class);

			urlSB = new StringBuilder();
			final List<InstrumentType> inst = searchResponse.getInstruments();
			for (final InstrumentType instrument : inst) {
				Logger.info(instrument.getName());
				if (name.equalsIgnoreCase(instrument.getName())) {
					final String urlType = InstrumentTypeEnum.getUrl(instrument.getInstrumentType());
					if (urlType != null) {
						urlSB.append(urlType).append('/').append(instrument.getUrlName());
					}
					break;
				}
			}

			if (urlSB.length() > 0) {
				urlSB.insert(0, MAIN_URL);
				url = new URL(urlSB.toString());
			}

		} catch (IOException e) {
			Logger.error(Settings.getI18n().tr("FfmWertpapierHistory.createSearchURLFailed"), e); //$NON-NLS-1$
			throw new FinanceException(
					Settings.getI18n().tr("FfmWertpapierHistory.createSearchURLFailed"), FinanceExceptionType.NETWORK, e); //$NON-NLS-1$
		}

		return url;
	}

	private List<HistoryValueBean> readData(final String instrumentID) throws FinanceException {

		final List<HistoryValueBean> wphv = new ArrayList<>();

		final StringBuilder urlString = new StringBuilder(DATA_URL).append(instrumentID).append(DATA_URL_PARAMS);
		URL url;
		try {
			url = new URL(urlString.toString());
			final String responseString = FfmWebAPI.getInstance().getResonseGETRequest(url, true);

			Logger.info(responseString);

			final ChartDataResponse dataResponse = FfmWertpapierHistory.MAPPER.readValue(responseString,
					ChartDataResponse.class);

			if (dataResponse.getInstruments().size() != EXACT_ONE) {
				throw new FinanceException(Settings.getI18n().tr("FfmWertpapierHistory.stockNotFound"), //$NON-NLS-1$
						FinanceExceptionType.DATA_NOT_FOUND);
			}

			final Instrument instrument = dataResponse.getInstruments().get(0);
			for (final ChartData data : instrument.getData()) {

				final HistoryValueBean historyBean = new HistoryValueBean(); // NOPMD
				historyBean.setDate(data.getDate());
				historyBean.setOpen(data.getOpen());
				historyBean.setClose(data.getAdjClose());
				historyBean.setAdjClose(data.getAdjClose());
				historyBean.setHigh(data.getHigh());
				historyBean.setLow(data.getLow());
				historyBean.setVolume(data.getVolume());
				historyBean.setIdString(name);

				wphv.add(historyBean);

			}

		} catch (IOException e) {
			Logger.error(Settings.getI18n().tr("FfmWertpapierHistory.dataURLWrong"), e); //$NON-NLS-1$
			throw new FinanceException(
					Settings.getI18n().tr("FfmWertpapierHistory.dataURLWrong"), FinanceExceptionType.DATA_NOT_FOUND, e); //$NON-NLS-1$
		}

		return wphv;

	}

}
