/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.frankmuenster.jameica.depot.finanzen.yahoo.finance;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.finanzen.FinanceException;
import de.frankmuenster.jameica.depot.finanzen.FinanceExceptionType;
import de.frankmuenster.jameica.depot.finanzen.HistoryValueBean;
import de.frankmuenster.jameica.depot.finanzen.IWertpapierHistory;
import de.willuhn.logging.Logger;

/**
 * Yahoo Wertpapierhistory Bean
 * 
 * @author munstfr
 * @version 1.0
 */
public class YahooWertpapierHistory implements IWertpapierHistory  {
	
	/**
	 * Der eindeutige Name
	 */
	private final transient String symbol;
	

	/**
	 * Konstruktor mit Angabe des Wertpapiers.
	 * 
	 * @param symbol
	 *            Das YAhoo-Symbol des Wertpapiers.
	 * @throws FinanceException
	 */
	public YahooWertpapierHistory(final String symbol) throws FinanceException {
		this.symbol = symbol;
	}

	/**
	 * @throws FinanceException
	 * 
	 */
	@Override
	public List<HistoryValueBean> loadData() throws FinanceException {

		String request = StringUtils.EMPTY;
		URL url;
		try {
			request = "https://ichart.yahoo.com/table.csv?s=" //$NON-NLS-1$
					+ URLEncoder.encode(symbol, "UTF-8"); //$NON-NLS-1$
			url = new URL(request);
			Logger.debug(request);
		} catch (MalformedURLException e1) {
			final String message = Settings.getI18n().tr(
					"YahooWertpapierHistory.malformedURL", request); //$NON-NLS-1$
			throw (FinanceException) new FinanceException(message,
					FinanceExceptionType.OTHER).initCause(e1);
		} catch (UnsupportedEncodingException e) {
			throw (FinanceException) new FinanceException(Settings
					.getI18n().tr("YahooWertpapierHistory.encodingNotSupported"), //$NON-NLS-1$
					FinanceExceptionType.OTHER).initCause(e);
		}

		final List<HistoryValueBean> wphvList = new ArrayList<>();
		try {
			final String responseString = YahooFinanceAPI.getInstance()
					.getResonseGETRequest(url, true);
			final BufferedReader bufReader = new BufferedReader(
					new StringReader(responseString));
			final DateFormat df1 = new SimpleDateFormat("yyyy-MM-dd", //$NON-NLS-1$
					Locale.GERMAN);
			final GregorianCalendar gregCalendar = new GregorianCalendar();

			// Lese Kopfzeile
			bufReader.readLine();
			// Und nun den Rest
			while (true) {
				final String line = bufReader.readLine();
				if (line == null) {
					break;
				}

				final String[] values = line.split(","); //$NON-NLS-1$

				final HistoryValueBean wphv = new HistoryValueBean(); // NOPMD
				wphv.setIdString(symbol);
				try {
					final Date date = df1.parse(values[0]);
					gregCalendar.setTime(date);
					gregCalendar.set(Calendar.HOUR_OF_DAY, 2);
					gregCalendar.set(Calendar.MINUTE, 0);
					gregCalendar.set(Calendar.SECOND, 0);
					wphv.setDate(gregCalendar.getTime());
				} catch (ParseException e) {
					Logger.warn(Settings.getI18n().tr(
							"YahooWertpapierHistory.parseDateError" //$NON-NLS-1$
							, e.getMessage()));
				}
				wphv.setOpen(Double.parseDouble(values[1]));
				wphv.setHigh(Double.parseDouble(values[2]));
				wphv.setLow(Double.parseDouble(values[3]));
				wphv.setClose(Double.parseDouble(values[4]));
				wphv.setVolume(Double.parseDouble(values[5]));
				wphv.setAdjClose(Double.parseDouble(values[6]));

				wphvList.add(wphv);

			}
		} catch (IOException e) {
			throw (FinanceException) new FinanceException(Settings
					.getI18n().tr("YahooWertpapierHistory.readingGeneralError"), //$NON-NLS-1$
					FinanceExceptionType.OTHER).initCause(e);
		}
		return wphvList;
	}
}
